/** HSpark automatically generated file **/

#include <vector>
#include "hspark.h"

namespace hspark {

class Sparkies {
public:
    Sparkies();
    

    void update(float dt);
    void render() const;
    

    Sphere bv() const;
    float age() const;

private:
    struct particle {
	float age;
	float3 position;
	float4 color;
	float3 velocity;
    };

    struct environment {
	float age;
    };

    std::vector<particle> particles;
    std::vector<int> dead_particles;
    environment env;
    Sphere sphere;
    typedef std::vector<particle>::iterator iterator;
    typedef std::vector<particle>::const_iterator const_iterator;
};

////////////////////////////////////////////////////////////

Sphere Sparkies::bv() const {
    return sphere;
}

inline float Sparkies::age() const {
    return env.age;
}

} /* end namespace hspark */

/** HSpark automatically generated file **/

#include "Sparkies.h"
#include <GL/gl.h>

namespace hspark {

Sparkies::Sparkies() {
    env.age = 0.0f;
    
    particles.reserve(256);
    dead_particles.reserve(256);
}

void Sparkies::update(float dt) {
    env.age += dt;
    
    float max_d = 0.0f;
    
    for(iterator part = particles.begin();
	part != particles.end();
	++part) {
	if (part->age < 0.0f) continue;
	
	/* Particle simulation */
	
	part->age += dt;
	
	if(part->age > 10.0f) {
	    dead_particles.push_back(part - particles.begin());
	    part->age = -1.0f;
	    continue;
	}
	
	part->position[0] = part->position[0] + part->velocity[0] * dt;
	part->position[1] = part->position[1] + part->velocity[1] * dt;
	part->position[2] = part->position[2] + part->velocity[2] * dt;
	
	part->velocity[1] = part->velocity[1] + -9.82f * dt;
	
	part->color[3] = part->color[3] + -0.1f * dt;
	
	max_d = max (max_d,
		     (1.0f +
		      part->position[0] * part->position[0] +
		      part->position[1] * part->position[1] +
		      part->position[2] * part->position[2]));
    }
    
    /* Emitter */ {
	/* Emitter interval not implemented yet */
    }
    
    /* Emitter */ {
	int n = (20.0f) * dt;
	
	for(int i=0; i<n; i++) {
	    iterator part;
	    if(dead_particles.empty()) {
		particles.push_back(particle());
		part = particles.begin() + particles.size() - 1;
	    } else {
		part = particles.begin() + dead_particles.back();
		dead_particles.pop_back();
	    }
	    
	    part->age = 0.0f;
	    
	    part->position[0] = 0.0f;
	    part->position[1] = 0.0f;
	    part->position[2] = 0.0f;
	    
	    part->velocity[0] = rand();
	    part->velocity[1] = rand();
	    part->velocity[2] = rand();
	    
	    part->color[0] = 1.0f;
	    part->color[1] = 0.0f;
	    part->color[2] = 0.0f;
	    part->color[3] = 0.0f;
	    
	    max_d = max (max_d,
			 (1.0f +
			  part->position[0] * part->position[0] +
			  part->position[1] * part->position[1] +
			  part->position[2] * part->position[2]));
	}
    }
    
    sphere.radius = max_d;
}

void Sparkies::render() const {
    glPointSize(1.0f);
    glBegin(GL_POINTS);
    
    for(const_iterator part = particles.begin();
	part != particles.end();
	++part) {
	glColor4f(part->color[0],
		  part->color[1],
		  part->color[2],
		  part->color[3]);
	glVertex3f(part->position[0],
		   part->position[1],
		   part->position[2]);
    }
    
    glEnd();
}

} /* end namespace hspark */


